@@ -27,15 +27,19 @@
 #include "libzc_private.h"
 #include "list.h"
 
-#define ZIP_SIG               0x04034b50
-#define ZIP_DATA_DESC_SIG     0x08074b50
-#define ZIP_STATIC_HEADER_LEN 30
+#define SIG                   0x04034b50
+#define DATA_DESC_SIG         0x08074b50
+#define STATIC_HEADER_LEN     30
 #define GP_BIT_HAS_DATA_DESC  (1 << 3)
 #define GP_BIT_ENCRYPTION     0x1
 #define MAX_FNLENGTH          (4096 + 255)
 
-/* zip file */
-struct header {
+struct zc_info {
+	/* begin and end of the entry payload */
+	long begin_offset;
+	long end_offset;
+
+	/* zip file entry info */
 	uint16_t version_needed;
 	uint16_t gen_bit_flag;
 	uint16_t comp_method;
@@ -47,17 +51,15 @@ struct header {
 	uint16_t filename_length;
 	uint16_t extra_field_length;
 	char *filename;
-};
 
-struct zc_info {
-	uint8_t enc_header[ZIP_ENCRYPTION_HEADER_LENGTH];
-	uint8_t magic;
+	/* encryption header */
+	struct zc_header header;
+	long header_offset;
+
+	/* zip file entry index */
 	int idx;
-	long enc_header_offset;
-	long begin_offset;
-	long end_offset;
-	struct header header;
-	struct list_head header_list;
+
+	struct list_head list;
 };
 
 /**
@@ -83,12 +85,15 @@ struct zc_file {
 
 static uint16_t get_le16_at(const uint8_t *b, size_t i)
 {
-	return b[i + 1] << 8 | b[i];
+	return (uint16_t)b[i + 1] << 8 | (uint16_t)b[i];
 }
 
 static uint32_t get_le32_at(const uint8_t *b, size_t i)
 {
-	return b[i + 3] << 24 | b[i + 2] << 16 | b[i + 1] << 8 | b[i];
+	return (uint32_t)b[i + 3] << 24 |
+		(uint32_t)b[i + 2] << 16 |
+		(uint32_t)b[i + 1] << 8 |
+		(uint32_t)b[i];
 }
 
 static bool is_encrypted(uint16_t flag)
@@ -111,40 +116,40 @@ static bool is_stored(uint16_t flag)
 	return flag == 0x0;
 }
 
-static uint8_t check_byte(const struct header *h)
+static uint8_t check_byte(const struct zc_info *e)
 {
-	if (has_data_desc(h->gen_bit_flag))
-		return h->last_mod_time >> 8;
-	return h->crc32 >> 24;
+	if (has_data_desc(e->gen_bit_flag))
+		return e->last_mod_time >> 8;
+	return e->crc32 >> 24;
 }
 
-static void clear_header_list(struct zc_file *f)
+static void clear_info_list(struct zc_file *f)
 {
 	struct zc_info *i, *tmp;
-	list_for_each_entry_safe(i, tmp, &f->info_head, header_list) {
-		list_del(&i->header_list);
-		free(i->header.filename);
+	list_for_each_entry_safe(i, tmp, &f->info_head, list) {
+		list_del(&i->list);
+		free(i->filename);
 		free(i);
 	}
 }
 
-static int fill_header_list(struct zc_file *f)
+static int fill_info_list(struct zc_file *f)
 {
 	int ret, sig, idx = 0;
-	uint8_t buf[ZIP_STATIC_HEADER_LEN - 4];
+	uint8_t buf[STATIC_HEADER_LEN - 4];
 	struct zc_info *info;
 
 	rewind(f->stream);
 
 	while (1) {
 
-		/* read zip header signature */
+		/* read zip entry signature */
 		ret = fread(buf, 4, 1, f->stream);
 		if (ret != 1)
 			return -1;
 
 		sig = get_le32_at(buf, 0);
-		if (sig != ZIP_SIG)
+		if (sig != SIG)
 			return idx == 0;
 
 		info = calloc(1, sizeof(struct zc_info));
@@ -152,53 +157,61 @@ static int fill_header_list(struct zc_file *f)
 			goto err1;
 
 		/* static header */
-		ret = fread(buf, ZIP_STATIC_HEADER_LEN - 4, 1, f->stream);
+		ret = fread(buf, STATIC_HEADER_LEN - 4, 1, f->stream);
 		if (ret != 1)
 			goto err2;
 
-		info->header.version_needed = get_le16_at(buf, 0);
-		info->header.gen_bit_flag = get_le16_at(buf, 2);
-		info->header.comp_method = get_le16_at(buf, 4);
-		info->header.last_mod_time = get_le16_at(buf, 6);
-		info->header.last_mod_date = get_le16_at(buf, 8);
-		info->header.crc32 = get_le32_at(buf, 10);
-		info->header.comp_size = get_le32_at(buf, 14);
-		info->header.uncomp_size = get_le32_at(buf, 18);
-		info->header.filename_length = get_le16_at(buf, 22);
-		info->header.extra_field_length = get_le16_at(buf, 24);
+		info->version_needed = get_le16_at(buf, 0);
+		info->gen_bit_flag = get_le16_at(buf, 2);
+		info->comp_method = get_le16_at(buf, 4);
+		info->last_mod_time = get_le16_at(buf, 6);
+		info->last_mod_date = get_le16_at(buf, 8);
+		info->crc32 = get_le32_at(buf, 10);
+		info->comp_size = get_le32_at(buf, 14);
+		info->uncomp_size = get_le32_at(buf, 18);
+		info->filename_length = get_le16_at(buf, 22);
+		info->extra_field_length = get_le16_at(buf, 24);
+
+		/* encrypted files should always have a minimum
+		 * compressed size of ENC_HEADER_LEN. See APPNOTE.txt
+		 * 4.4.8. */
+		if (is_encrypted(info->gen_bit_flag) &&
+		    info->comp_size < ENC_HEADER_LEN) {
+			goto err2;
+		}
 
 		/* filename (variable length) */
-		if (!info->header.filename_length ||
-		    info->header.filename_length > MAX_FNLENGTH)
+		if (!info->filename_length ||
+		    info->filename_length > MAX_FNLENGTH)
 			goto err2;
 
-		info->header.filename = calloc(1, info->header.filename_length + 1);
-		if (!info->header.filename)
+		info->filename = calloc(1, info->filename_length + 1);
+		if (!info->filename)
 			goto err2;
 
-		ret = fread(info->header.filename, info->header.filename_length, 1, f->stream);
+		ret = fread(info->filename, info->filename_length, 1, f->stream);
 		if (ret != 1)
 			goto err2;
 
 		/* skip the extra field */
-		ret = fseek(f->stream, info->header.extra_field_length, SEEK_CUR);
+		ret = fseek(f->stream, info->extra_field_length, SEEK_CUR);
 		if (ret < 0)
 			goto err2;
 
 		/* set offsets and read encrypted header */
-		if (is_encrypted(info->header.gen_bit_flag)) {
-			info->magic = check_byte(&info->header);
-			info->enc_header_offset = ftell(f->stream);
-			info->begin_offset = info->enc_header_offset + ZIP_ENCRYPTION_HEADER_LENGTH;
-			info->end_offset = info->enc_header_offset + info->header.comp_size;
-			ret = fread(info->enc_header, ZIP_ENCRYPTION_HEADER_LENGTH, 1, f->stream);
+		if (is_encrypted(info->gen_bit_flag)) {
+			info->header.magic = check_byte(info);
+			info->header_offset = ftell(f->stream);
+			info->begin_offset = info->header_offset + ENC_HEADER_LEN;
+			info->end_offset = info->header_offset + info->comp_size;
+			ret = fread(info->header.buf, ENC_HEADER_LEN, 1, f->stream);
 			if (ret != 1)
 				goto err2;
 		} else {
-			info->magic = 0;
-			info->enc_header_offset = -1;
+			info->header.magic = 0;
+			info->header_offset = -1;
 			info->begin_offset = ftell(f->stream);
-			info->end_offset = info->begin_offset + info->header.comp_size;
+			info->end_offset = info->begin_offset + info->comp_size;
 		}
 
 		/* seek to end of compressed stream */
@@ -207,7 +220,7 @@ static int fill_header_list(struct zc_file *f)
 			goto err2;
 
 		/* seek data descriptor signature */
-		if (has_data_desc(info->header.gen_bit_flag)) {
+		if (has_data_desc(info->gen_bit_flag)) {
 			int data_desc_sig;
 
 			/*
@@ -221,24 +234,24 @@ static int fill_header_list(struct zc_file *f)
 			if (ret != 1)
 				goto err2;
 			data_desc_sig = get_le32_at(buf, 0);
-			ret = fseek(f->stream, data_desc_sig == ZIP_DATA_DESC_SIG ? 12 : 8, SEEK_CUR);
+			ret = fseek(f->stream, data_desc_sig == DATA_DESC_SIG ? 12 : 8, SEEK_CUR);
 			if (ret)
 				goto err2;
 		}
 
 		info->idx = idx;
 
-		list_add_tail(&info->header_list, &f->info_head);
+		list_add_tail(&info->list, &f->info_head);
 		++idx;
 	}
 
 	return 0;
 
 err2:
-	free(info->header.filename);
+	free(info->filename);
 	free(info);
 err1:
-	clear_header_list(f);
+	clear_info_list(f);
 	return -1;
 }
 
@@ -316,7 +329,7 @@ ZC_EXPORT int zc_file_open(struct zc_file *file)
 
 	stream = fopen(file->filename, "r");
 	if (!stream) {
-		err(file->ctx, "open() failed: %s.\n", strerror(errno));
+		err(file->ctx, "fopen() failed: %s.\n", strerror(errno));
 		return -1;
 	}
 
@@ -324,7 +337,7 @@ ZC_EXPORT int zc_file_open(struct zc_file *file)
 
 	file->stream = stream;
 
-	if (fill_header_list(file)) {
+	if (fill_info_list(file)) {
 		err(file->ctx, "failure while reading headers.\n");
 		goto err;
 	}
@@ -349,7 +362,7 @@ ZC_EXPORT int zc_file_close(struct zc_file *file)
 	if (!zc_file_isopened(file))
 		return -1;
 
-	clear_header_list(file);
+	clear_info_list(file);
 
 	if (fclose(file->stream)) {
 		err(file->ctx, "fclose() failed: %s.\n", strerror(errno));
@@ -373,10 +386,19 @@ ZC_EXPORT bool zc_file_isopened(struct zc_file *file)
 	return (file->stream != NULL);
 }
 
+static bool consider_file(struct zc_info *info)
+{
+	if (!is_encrypted(info->gen_bit_flag) ||
+	    (!is_deflated(info->comp_method) &&
+	     !is_stored(info->comp_method)))
+		return false;
+	return true;
+}
+
 /**
- * read_validation_data:
+ * read_zc_header:
  *
- * Read the validation data from the file and store them in the vdata
+ * Read the validation data from the file and store them in the header
  * array. At most nmemb elements will be stored in the array.
  *
  * The file must be opened before calling this function.
@@ -384,29 +406,23 @@ ZC_EXPORT bool zc_file_isopened(struct zc_file *file)
  * @retval 0  No encryption data found in this file.
  * @retval >0 The number of encryption data objects read.
  */
-size_t read_validation_data(struct zc_file *file, struct validation_data *vdata,
-			    size_t nmemb)
+size_t read_zc_header(struct zc_file *file, struct zc_header *h, size_t len)
 {
 	struct zc_info *info;
-	size_t valid_files = 0;
+	size_t valid = 0;
 
-	list_for_each_entry(info, &file->info_head, header_list) {
-		if (!is_encrypted(info->header.gen_bit_flag))
-			continue;
-		if (!is_deflated(info->header.comp_method) &&
-		    !is_stored(info->header.comp_method))
+	list_for_each_entry(info, &file->info_head, list) {
+		if (!consider_file(info))
 			continue;
 
-		vdata[valid_files].magic = info->magic;
-		memcpy(vdata[valid_files].encryption_header,
-		       info->enc_header,
-		       ZIP_ENCRYPTION_HEADER_LENGTH);
+		h[valid].magic = info->header.magic;
+		memcpy(h[valid].buf, info->header.buf, ENC_HEADER_LEN);
 
-		if (++valid_files == nmemb)
+		if (++valid == len)
 			break;
 	}
 
-	return valid_files;
+	return valid;
 }
 
 static struct zc_info *find_file_smallest(struct zc_file *file)
@@ -414,13 +430,9 @@ static struct zc_info *find_file_smallest(struct zc_file *file)
 	struct zc_info *info, *ret = NULL;
 	long s = LONG_MAX;
 
-	list_for_each_entry(info, &file->info_head, header_list) {
-		if (!is_encrypted(info->header.gen_bit_flag))
+	list_for_each_entry(info, &file->info_head, list) {
+		if (!consider_file(info))
 			continue;
-		if (!is_deflated(info->header.comp_method) &&
-		    !is_stored(info->header.comp_method))
-			continue;
-
 		long tmp = info->end_offset - info->begin_offset;
 		if (tmp < s) {
 			s = tmp;
@@ -442,9 +454,9 @@ int read_crypt_data(struct zc_file *file, unsigned char **buf,
 	if (!info)
 		return -1;
 
-	to_read = info->end_offset - info->enc_header_offset;
+	to_read = info->end_offset - info->header_offset;
 
-	err = fseek(file->stream, info->enc_header_offset, SEEK_SET);
+	err = fseek(file->stream, info->header_offset, SEEK_SET);
 	if (err) {
 		err(file->ctx, "fseek(): %s\n", strerror(errno));
 		return -1;
@@ -467,8 +479,8 @@ int read_crypt_data(struct zc_file *file, unsigned char **buf,
 
 	*buf = tmp;
 	*len = ret;
-	*original_crc = info->header.crc32;
-	*deflated = is_deflated(info->header.comp_method);
+	*original_crc = info->crc32;
+	*deflated = is_deflated(info->comp_method);
 
 	return 0;
 
@@ -483,41 +495,51 @@ ZC_EXPORT struct zc_info *zc_file_info_next(struct zc_file *file,
 	struct zc_info *i;
 
 	if (!info) {
-		i = list_entry((&file->info_head)->next, typeof(*i), header_list);
+		i = list_entry(file->info_head.next, struct zc_info, list);
 		return i;
 	}
 
-	i = list_entry(info->header_list.next, struct zc_info, header_list);
-
-	if (&i->header_list == &file->info_head)
+	if (info->list.next == &file->info_head)
 		return NULL;
 
+	i = list_entry(info->list.next, struct zc_info, list);
+
 	return i;
 }
 
 ZC_EXPORT const char *zc_file_info_name(const struct zc_info *info)
 {
-	return info->header.filename;
+	return info->filename;
 }
 
 ZC_EXPORT uint32_t zc_file_info_size(const struct zc_info *info)
 {
-	return info->header.uncomp_size;
+	return info->uncomp_size;
 }
 
-ZC_EXPORT long zc_file_info_offset(const struct zc_info *info)
+ZC_EXPORT uint32_t zc_file_info_compressed_size(const struct zc_info *info)
+{
+	return info->comp_size;
+}
+
+ZC_EXPORT long zc_file_info_offset_begin(const struct zc_info *info)
 {
 	return info->begin_offset;
 }
 
+ZC_EXPORT long zc_file_info_offset_end(const struct zc_info *info)
+{
+	return info->end_offset;
+}
+
 ZC_EXPORT long zc_file_info_crypt_header_offset(const struct zc_info *info)
 {
-	return info->enc_header_offset;
+	return info->header_offset;
 }
 
 ZC_EXPORT const uint8_t *zc_file_info_enc_header(const struct zc_info *info)
 {
-	return info->enc_header;
+	return info->header.buf;
 }
 
 ZC_EXPORT int zc_file_info_idx(const struct zc_info *info)
