@@ -19,79 +19,85 @@
 #include <check.h>
 #include <stdlib.h>
 
+#include "libzc.h"
 #include "ptext_private.h"
 #include "test_plaintext.h"
+#include "ptext_reduce.c"
 
 #define KEY3(index) test_plaintext[index] ^ test_ciphertext[index]
 
-struct key2r *k2r;
+struct zc_ctx *ctx;
+struct zc_crk_ptext *ptext;
 
-void setup_key2r()
+void setup_reduce()
 {
-	key2r_new(&k2r);
+	zc_new(&ctx);
+	zc_crk_ptext_new(ctx, &ptext);
 }
 
-void teardown_key2r()
+void teardown_reduce()
 {
-	key2r_free(k2r);
+	zc_crk_ptext_unref(ptext);
+	zc_unref(ctx);
 }
 
 START_TEST(test_can_get_bits_15_2)
 {
-	uint16_t *bits15_2;
-	bits15_2 = key2r_get_bits_15_2(k2r, 0);
-	fail_if(bits15_2[0] != 0);
+	fail_if(ptext->bits_15_2[0] != 0);
 }
 END_TEST
 
 START_TEST(test_can_generate_first_gen_key2)
 {
-	struct ka *key2_first_gen;
+	uint32_t *key2_first_gen;
 	uint16_t *bits15_2;
 
-	bits15_2 = key2r_get_bits_15_2(k2r, 0);
-	key2_first_gen = key2r_compute_first_gen(bits15_2);
-	fail_if(ka_at(key2_first_gen, 0) != 0);
-	ka_free(key2_first_gen);
+	bits15_2 = get_bits_15_2(ptext->bits_15_2, 0);
+	key2_first_gen = calloc((1 << 22), sizeof(uint32_t));
+	generate_all_key2_bits_31_2(key2_first_gen, bits15_2);
+	fail_if(key2_first_gen[0] != 0);
+	free(key2_first_gen);
 }
 END_TEST
 
+#ifdef EXTRACHECK
 START_TEST(test_can_generate_next_array_from_plaintext)
 {
-	struct ka *key2_first_gen;
-	struct ka *key2_next_gen;
+	uint32_t *key2_first_gen, *key2_next_gen;
+	size_t key2_first_gen_size, total = 0;
 
 	uint8_t key3i = KEY3(TEST_PLAINTEXT_SIZE - 1);
 	uint8_t key3im1 = KEY3(TEST_PLAINTEXT_SIZE - 2);
 	uint8_t key3im2 = KEY3(TEST_PLAINTEXT_SIZE - 3);
 
-	key2_first_gen = key2r_compute_first_gen(key2r_get_bits_15_2(k2r, key3i));
-	ka_alloc(&key2_next_gen, pow2(22));
+	key2_first_gen = calloc((1 << 22), sizeof(uint32_t));
+	generate_all_key2_bits_31_2(key2_first_gen, get_bits_15_2(ptext->bits_15_2, key3i));
+	key2_first_gen_size = (1 << 22);
+	key2_next_gen = calloc((1 << 22), sizeof(uint32_t));
 
-	ka_empty(key2_next_gen);
-	for (uint32_t i = 0; i < key2_first_gen->size; ++i) {
-		fail_if(key2r_compute_single(ka_at(key2_first_gen, i),
-					     key2_next_gen,
-					     key2r_get_bits_15_2(k2r, key3im1),
-					     key2r_get_bits_15_2(k2r, key3im2),
-					     KEY2_MASK_6BITS) != 0);
-	}
+	for (size_t i = 0; i < key2_first_gen_size; ++i)
+		total += key2r_compute_single(key2_first_gen[i],
+					      &key2_next_gen[total],
+					      get_bits_15_2(ptext->bits_15_2, key3im1),
+					      get_bits_15_2(ptext->bits_15_2, key3im2),
+					      KEY2_MASK_6BITS);
 
-	ka_uniq(key2_next_gen);
+	uniq(key2_next_gen, &total);
 
-	fail_if(key2_next_gen->size != 2256896);
+	fail_if(total != 2256896);
 
-	ka_free(key2_next_gen);
-	ka_free(key2_first_gen);
+	free(key2_next_gen);
+	free(key2_first_gen);
 }
 END_TEST
+#endif
 
 Suite *reduce_suite()
 {
 	Suite *s = suite_create("reduce");
 
 	TCase *tc_core = tcase_create("Core");
-	tcase_add_checked_fixture(tc_core, setup_key2r, teardown_key2r);
+	tcase_add_checked_fixture(tc_core, setup_reduce, teardown_reduce);
 	tcase_add_test(tc_core, test_can_get_bits_15_2);
 	tcase_add_test(tc_core, test_can_generate_first_gen_key2);
 #ifdef EXTRACHECK
