@@ -29,13 +29,16 @@
 #include <sys/mman.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <sys/time.h>
 
 #include "yazc.h"
 #include "libzc.h"
 
-static const char short_opts[] = "t:h";
+static const char short_opts[] = "t:oSh";
 static const struct option long_opts[] = {
 	{"threads", required_argument, 0, 't'},
+	{"offset", no_argument, 0, 'o'},
+	{"stats", no_argument, 0, 'S'},
 	{"help", no_argument, 0, 'h'},
 	{NULL, 0, 0, 0}
 };
@@ -47,38 +50,47 @@ struct filed {
 	off_t txt_begin;            /* begin offset of the plain or cipher text */
 	off_t txt_end;              /* end offset of the plain or cipher text */
 	off_t file_begin;           /* offset of the first byte for the
-                                  file we are using in the encrypted
-                                  archive */
+                                       file we are using in the encrypted
+                                       archive */
 	void *map;
 };
 
 static struct filed cipher = {NULL, 0, 0, 0, -1, NULL};
 static struct filed plain = {NULL, 0, 0, 0, -1, NULL};
 static long thread_count;
+static bool stats = false;
 
-static void print_help(const char *name)
+static void usage(const char *name)
 {
 	fprintf(stderr,
 		"Usage:\n"
-		"\t%s [options] PLAIN:OFF1:OFF2 CIPHER:OFF1:OFF2:BEGIN\n"
+		"\t%s [options] PLAIN ENTRY CIPHER ENTRY\n"
+		"\t%s [options] -o PLAIN OFFSETS CIPHER OFFSETS BEGIN\n"
 		"\n"
 		"The plaintext subcommand uses a known vulnerability in the pkzip\n"
 		"stream cipher to find the internal representation of the encryption\n"
 		"key. To use this attack type you need at least 13 known plaintext\n"
 		"bytes from any file in the archive.\n"
 		"\n"
-		"Example:\n"
-		"\t %s plain.bin:100:650 archive.zip:112:662:64\n"
+		"Example 1:\n"
+		"\t %s -o plain.zip 100 650 cipher.zip 112 662 64\n"
 		"\n"
 		"Use plaintext bytes 100 to 650 and map them to ciphertext bytes\n"
 		"112 to 662. Use these bytes to reduce the number of keys and perform\n"
 		"the attack. Once the intermediate key is found, decrypt the rest of\n"
 		"the cipher (begins at offset 64) to get the internal representation.\n"
 		"\n"
+		"Example 2:\n"
+		"\t %s plain.zip file1.bin cipher.zip file2.bin\n"
+		"\n"
+		"Use plaintext bytes from the file1.bin entry in plain.zip and map them\n"
+		"to file2.bin from cipher.zip.\n"
 		"Options:\n"
 		"\t-t, --threads=NUM       spawn NUM threads\n"
+		"\t-o, --offset            use offsets instead of entry names\n"
+		"\t-S, --stats             print statistics\n"
 		"\t-h, --help              show this help\n",
-		name, name);
+		name, name, name, name);
 }
 
 static int parse_offset(const char *tok, off_t *offset)
@@ -96,38 +108,105 @@ static int parse_offset(const char *tok, off_t *offset)
 	return 0;
 }
 
-static int parse_opt(char *opt, int count, const char **filename, off_t *off1,
-		     off_t *off2, off_t *off3)
+enum text_src {
+	SRC_PLAIN = 0,
+	SRC_CIPHER,
+	SRC_NUM
+};
+
+static int parse_entry_opts(char *argv[])
 {
-	char *saveptr = NULL, *token;
-	int err = -1;
+	struct zc_ctx *ctx;
+	struct zc_file *f;
+	char *filename, *entry;
+	int err = 0, matches = 0;
 
-	if (!opt)
+	if (zc_new(&ctx))
 		return -1;
 
-	token = strtok_r(opt, ":", &saveptr);
-	if (!token)
-		return -1;
-	*filename = token;
+	for (int src = SRC_PLAIN; src < SRC_NUM; ++src) {
+		filename = argv[optind++];
+		entry = argv[optind++];
 
-	for (int i = 0; i < count; ++i) {
-		token = strtok_r(NULL, ":", &saveptr);
-		if (!token)
-			return -1;
-		switch (i) {
-		case 0:
-			err = parse_offset(token, off1);
-			break;
-		case 1:
-			err = parse_offset(token, off2);
-			break;
-		case 2:
-			err = parse_offset(token, off3);
-			break;
-		}
+		dbg("%s: %s %s\n",
+		    src == SRC_PLAIN ? "plaintext" : "ciphertext",
+		    filename,
+		    entry);
+
+		err = zc_file_new_from_filename(ctx, filename, &f);
 		if (err)
-			return -1;
+			goto err1;
+
+		err = zc_file_open(f);
+		if (err) {
+			zc_file_unref(f);
+			goto err1;
+		}
+
+		struct zc_info *info = zc_file_info_next(f, NULL);
+		while (info) {
+			if (strcmp(zc_file_info_name(info), entry) != 0)
+				/* filenames do not match */
+				goto next;
+
+			if ((src == SRC_PLAIN && zc_file_info_crypt_header_offset(info) != -1) ||
+			    (src == SRC_CIPHER && zc_file_info_crypt_header_offset(info) == -1))
+				/* plaintext is encrypted or ciphertext is not encrypted ? */
+				goto next;
+
+			/* found match */
+			struct filed *fd = src == SRC_PLAIN ? &plain : &cipher;
+			fd->txt_begin = zc_file_info_offset_begin(info);
+			fd->txt_end = zc_file_info_offset_end(info);
+			fd->file_begin = zc_file_info_crypt_header_offset(info);
+			fd->name = filename;
+			matches++;
+			dbg("found match: %s %lld %lld %lld\n",
+			    entry,
+			    (long long)fd->txt_begin,
+			    (long long)fd->txt_end,
+			    (long long)fd->file_begin);
+			break;
+		next:
+			dbg("skipping %s\n", zc_file_info_name(info));
+			info = zc_file_info_next(f, info);
+			continue;
+		}
+		zc_file_close(f);
+		zc_file_unref(f);
 	}
+err1:
+	zc_unref(ctx);
+	if (err)
+		return err;
+	return matches == 2 ? 0 : -1;
+}
+
+static int parse_offset_opts(char *argv[])
+{
+	plain.name = argv[optind++];
+	if (parse_offset(argv[optind++], &plain.txt_begin))
+	    return -1;
+	if (parse_offset(argv[optind++], &plain.txt_end))
+	    return -1;
+
+	cipher.name = argv[optind++];
+	if (parse_offset(argv[optind++], &cipher.txt_begin))
+	    return -1;
+	if (parse_offset(argv[optind++], &cipher.txt_end))
+	    return -1;
+	if (parse_offset(argv[optind], &cipher.file_begin))
+	    return -1;
+
+	dbg("plaintext: %s %lld %lld\n",
+	    plain.name,
+	    (long long)plain.txt_begin,
+	    (long long)plain.txt_end);
+	dbg("ciphertext: %s %lld %lld %lld\n",
+	    cipher.name,
+	    (long long)cipher.txt_begin,
+	    (long long)cipher.txt_end,
+	    (long long)cipher.file_begin);
 
 	return 0;
 }
@@ -167,23 +246,28 @@ static int mmap_text_buf(struct filed *file)
 
 	fd = open(file->name, O_RDONLY);
 	if (fd < 0) {
-		yazc_err("open() failed: %s.\n", strerror(errno));
+		err("open() failed: %s.\n", strerror(errno));
 		return -1;
 	}
 
 	if (fstat(fd, &filestat) < 0) {
-		yazc_err("fstat() failed: %s.\n", strerror(errno));
+		err("fstat() failed: %s.\n", strerror(errno));
 		goto error;
 	}
 
 	if (filestat.st_size == 0) {
-		yazc_err("file %s is empty.\n", file->name);
+		err("file %s is empty.\n", file->name);
+		goto error;
+	}
+
+	if (file->txt_end >= filestat.st_size) {
+		err("end offset (%lld) goes past the end of the file.\n", (long long)file->txt_end);
 		goto error;
 	}
 
 	map = mmap(NULL, filestat.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
 	if (map == MAP_FAILED) {
-		yazc_err("mmap() failed: %s.\n", strerror(errno));
+		err("mmap() failed: %s.\n", strerror(errno));
 		goto error;
 	}
 
@@ -194,23 +278,25 @@ static int mmap_text_buf(struct filed *file)
 
 error:
 	if (close(fd))
-		yazc_err("close() failed: %s\n", strerror(errno));
+		err("close() failed: %s\n", strerror(errno));
 	return -1;
 }
 
 static int unmap_text_buf(struct filed *file)
 {
 	if (munmap(file->map, size_of_map(file)) < 0)
-		yazc_err("munmap() failed: %s.\n", strerror(errno));
+		err("munmap() failed: %s.\n", strerror(errno));
 	if (close(file->fd))
-		yazc_err("close() failed: %s.\n", strerror(errno));
+		err("close() failed: %s.\n", strerror(errno));
 	return 0;
 }
 
 static int do_plaintext(int argc, char *argv[])
 {
 	const char *arg_threads = NULL;
+	bool arg_use_offsets = false;
 	struct zc_crk_ptext *ptext;
+	struct timeval begin, end;
 	int err = 0;
 
 	for (;;) {
@@ -219,70 +305,81 @@ static int do_plaintext(int argc, char *argv[])
 		if (c == -1)
 			break;
 		switch (c) {
+		case 'o':
+			arg_use_offsets = true;
+			break;
 		case 't':
 			arg_threads = optarg;
 			break;
+		case 'S':
+			stats = true;
+			break;
 		case 'h':
-			print_help(basename(argv[0]));
+			usage(basename(argv[0]));
 			return EXIT_SUCCESS;
 		default:
-			yazc_err("unexpected getopt_long() value '%c'.\n", c);
+			err("unexpected getopt_long() value '%c'.\n", c);
 			return EXIT_FAILURE;
 		}
 	}
 
-	if (optind >= argc) {
-		yazc_err("missing arguments.\n");
-		print_help(basename(argv[0]));
-		return EXIT_FAILURE;
-	}
+	if (optind >= argc)
+		goto missing;
 
 	/* number of concurrent threads */
 	if (arg_threads) {
 		thread_count = atol(arg_threads);
 		if (thread_count < 1) {
-			yazc_err("number of threads can't be less than one.\n");
+			err("number of threads can't be less than one.\n");
 			return EXIT_FAILURE;
 		}
 	} else
 		thread_count = -1;	/* auto */
 
-	if (parse_opt(argv[optind], 2, &plain.name,
-		      &plain.txt_begin, &plain.txt_end, NULL) < 0) {
-		yazc_err("parsing plaintext file offsets failed.\n");
-		return EXIT_FAILURE;
-	}
+	if (arg_use_offsets) {
+		/* parse raw offsets */
+		if (argc - optind < 7)
+			goto missing;
 
-	if (parse_opt(argv[optind + 1], 3, &cipher.name,
-		      &cipher.txt_begin, &cipher.txt_end, &cipher.file_begin) < 0) {
-		yazc_err("parsing cipher file offsets failed.\n");
-		return EXIT_FAILURE;
+		if (parse_offset_opts(argv)) {
+			err("error parsing offsets.\n");
+			return EXIT_FAILURE;
+		}
+	} else {
+		/* get offsets from entry names */
+		if (argc - optind < 4)
+			goto missing;
+
+		if (parse_entry_opts(argv)) {
+			err("error parsing entries.\n");
+			return EXIT_FAILURE;
+		}
 	}
 
 	if (!validate_offsets()) {
-		yazc_err("offsets validation failed.\n");
+		err("offsets validation failed.\n");
 		return EXIT_FAILURE;
 	}
 
 	if (mmap_text_buf(&plain) < 0) {
-		yazc_err("mapping plaintext data failed.\n");
+		err("mapping plaintext data failed.\n");
 		return EXIT_FAILURE;
 	}
 
 	if (mmap_text_buf(&cipher) < 0) {
-		yazc_err("mapping ciphertext data failed.\n");
+		err("mapping ciphertext data failed.\n");
 		goto error1;
 	}
 
 	zc_new(&ctx);
 	if (!ctx) {
-		yazc_err("zc_new() failed!\n");
+		err("zc_new() failed!\n");
 		goto error2;
 	}
 
 	err = zc_crk_ptext_new(ctx, &ptext);
 	if (err < 0) {
-		yazc_err("zc_crk_ptext_new() failed!\n");
+		err("zc_crk_ptext_new() failed!\n");
 		goto error3;
 	}
 
@@ -291,7 +388,7 @@ static int do_plaintext(int argc, char *argv[])
 				    &((const uint8_t *)cipher.map)[cipher.txt_begin],
 				    size_of_map(&plain));
 	if (err < 0) {
-		yazc_err("zc_crk_ptext_set_text() failed!\n");
+		err("zc_crk_ptext_set_text() failed!\n");
 		goto error4;
 	}
 
@@ -299,10 +396,11 @@ static int do_plaintext(int argc, char *argv[])
 
 	printf("Key2 reduction...");
 	fflush(stdout);
+	gettimeofday(&begin, NULL);
 	err = zc_crk_ptext_key2_reduction(ptext);
 	if (err < 0) {
 		printf("\n");
-		yazc_err("reducing key2 candidates failed.\n");
+		err("reducing key2 candidates failed.\n");
 		goto error4;
 	}
 	printf(" done! %zu keys found.\n", zc_crk_ptext_key2_count(ptext));
@@ -313,7 +411,7 @@ static int do_plaintext(int argc, char *argv[])
 	err = zc_crk_ptext_attack(ptext, &out_key);
 	if (err < 0) {
 		printf("\n");
-		yazc_err("attack failed! Wrong plaintext?\n");
+		err("attack failed! Wrong plaintext?\n");
 		goto error4;
 	}
 	printf(" done!\n");
@@ -327,7 +425,7 @@ static int do_plaintext(int argc, char *argv[])
 					     cipher.txt_begin - cipher.file_begin,
 					     &int_rep);
 	if (err < 0) {
-		yazc_err("finding internal representation failed.\n");
+		err("finding internal representation failed.\n");
 		goto error4;
 	}
 
@@ -339,11 +437,13 @@ static int do_plaintext(int argc, char *argv[])
 	char pw[14];
 	err = zc_crk_ptext_find_password(ptext, &int_rep, pw, sizeof(pw));
 	if (err < 0) {
-		yazc_err(" failed!\n");
+		err(" failed!\n");
 		err = EXIT_FAILURE;
 		goto error4;
 	}
 
+	gettimeofday(&end, NULL);
+
 	printf("\nOriginal password: ");
 	for (int i = 0; i < err; ++i) {
 		if (isprint(pw[i]))
@@ -353,6 +453,10 @@ static int do_plaintext(int argc, char *argv[])
 	}
 	printf("\n");
 
+	if (stats)
+		printf("Runtime: %f secs.\n", (double)(end.tv_usec - begin.tv_usec) / 1000000 +
+		       (double)(end.tv_sec - begin.tv_sec));
+
 	err = EXIT_SUCCESS;
 
 error4:
@@ -364,6 +468,11 @@ error2:
 error1:
 	unmap_text_buf(&plain);
 	return err;
+
+missing:
+	err("missing argument.\n");
+	usage(basename(argv[0]));
+	return EXIT_FAILURE;
 }
 
 const struct yazc_cmd yazc_cmd_plaintext = {
